<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 1999-2003 Kasper Skrhj (kasper@typo3.com)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Miscellaneous functions for general purpose.
 *
 * The class is intended to be used without creating an instance of it.
 * So use t3lib_div::[method-name] to refer to the functions, eg. 't3lib_div::milliseconds()'
 *
 * @author	Kasper Skrhj <kasper@typo3.com>
 */


class t3lib_div {


	## *************************
	## BASIC FUNCTIONS
	## *************************


	/**
	 * milliseconds
	 *
	 * microtime recalculated to t3lib_div::milliseconds(1/1000 sec)
	 *
	 * @return 	integer
	 */
	function milliseconds()	{
		$p=explode(" ",microtime());
		return round(($p[0]+$p[1])*1000);
	}
	
	/**
	 * Sets global variables from HTTP_POST_VARS or HTTP_GET_VARS
	 */
	function setGPvars($list,$strip=0)	{
		$vars = t3lib_div::trimExplode(",",$list,1);
		while(list(,$var)=each($vars))	{
			$GLOBALS[$var] = t3lib_div::GPvar($var,$strip);
		}
	}

	/**
	 * GET/POST variable
	 *
	 * Returns the 'GLOBAL' value of incoming data from POST or GET, with priority to POST (that is equalent to 'GP' order)
	 * Strips slashes of string-outputs, but not arrays UNLESS $strip is set. If $strip is set all output will have escaped characters unescaped.
	 */
	function GPvar($var,$strip=0)	{
		$value = isset($GLOBALS["HTTP_POST_VARS"][$var]) ? $GLOBALS["HTTP_POST_VARS"][$var] : $GLOBALS["HTTP_GET_VARS"][$var];
		if (isset($value) && is_string($value))	{$value=stripslashes($value);}	// Originally check "&& get_magic_quotes_gpc() " but the values of HTTP_GET_VARS are always slashed regardless of get_magic_quotes_gpc() because HTTP_POST/GET_VARS are run through addSlashesOnArray in the very beginning of index_ts.php eg.
		if ($strip && isset($value) && is_array($value)) {t3lib_div::stripSlashesOnArray($value);}
		return $value;
	}

	/**
	 * Returns the GET/POST global arrays merged with POST taking precedence.
	 */
	function GParrayMerged($var)	{
		$postA = is_array($GLOBALS["HTTP_POST_VARS"][$var]) ? $GLOBALS["HTTP_POST_VARS"][$var] : array();
		$getA = is_array($GLOBALS["HTTP_GET_VARS"][$var]) ? $GLOBALS["HTTP_GET_VARS"][$var] : array();
		$mergedA = t3lib_div::array_merge_recursive_overrule($getA,$postA);
		t3lib_div::stripSlashesOnArray($mergedA);
		return $mergedA;
	}

	/**
	 * Client Browser Information
	 *
	 * @return 	string 	false (bool) on non-4+ browsers, "msie" if browser is IE, "net" if browser is navigator/mozilla, "opera" if opera, "konqu" if konqueror
	 */
	function clientInfo($useragent="")	{
		if (!$useragent) $useragent=t3lib_div::getIndpEnv("HTTP_USER_AGENT");

		$bInfo=array();
			// Which browser?
		if (strstr($useragent,"Konqueror"))	{
			$bInfo["BROWSER"]= "konqu";
		} elseif (strstr($useragent,"Opera"))	{
			$bInfo["BROWSER"]= "opera";
		} elseif (strstr($useragent,"MSIE 4") || strstr($useragent,"MSIE 5") || strstr($useragent,"MSIE 6"))	{
			$bInfo["BROWSER"]= "msie";
		} elseif (strstr($useragent,"Mozilla/4") || strstr($useragent,"Mozilla/5"))	{
			$bInfo["BROWSER"]="net";
		}
		if ($bInfo["BROWSER"])	{
				// Browser version
			switch($bInfo["BROWSER"])	{
				case "net":
					$bInfo["VERSION"]= doubleval(substr($useragent,8));
					if (strstr($useragent,"Netscape6/")) {$bInfo["VERSION"]=doubleval(substr(strstr($useragent,"Netscape6/"),10));}
					if (strstr($useragent,"Netscape/7")) {$bInfo["VERSION"]=doubleval(substr(strstr($useragent,"Netscape/7"),9));}
				break;
				case "msie":
					$tmp = strstr($useragent,"MSIE");
					$bInfo["VERSION"] = doubleval(substr($tmp,4));
				break;
				case "opera":
					$tmp = strstr($useragent,"Opera");
					$bInfo["VERSION"] = doubleval(substr($tmp,5));
				break;
				case "konqu":
					$tmp = strstr($useragent,"Konqueror/");
					$bInfo["VERSION"] = doubleval(substr($tmp,10));
				break;
			}
				// Client system
			if (strstr($useragent,"Win"))	{
				$bInfo["SYSTEM"] = "win";
			} elseif (strstr($useragent,"Mac"))	{
				$bInfo["SYSTEM"] = "mac";
			} elseif (strstr($useragent,"Linux") || strstr($useragent,"X11") || strstr($useragent,"SGI") || strstr($useragent," SunOS ") || strstr($useragent," HP-UX "))	{
				$bInfo["SYSTEM"] = "unix";
			}
		}
			// Is true if the browser supports css to format forms, especially the width
		$bInfo["FORMSTYLE"]=($bInfo["BROWSER"]=="msie" || ($bInfo["BROWSER"]=="net"&&$bInfo["VERSION"]>=5) || $bInfo["BROWSER"]=="opera" || $bInfo["BROWSER"]=="konqu");
#debug(array($useragent,$bInfo));
		return $bInfo;
	}

	
	
	
	
	
	
	


	## ************************
	## IMAGE FUNCTIONS
	## ************************
	
	
	/**
	 * Returns the GET/POST global arrays merged with POST taking precedence.
	 */
	function gif_compress($theFile, $type)	{
		// This function is a workaround for the fact that ImageMagick and/or GD does not compress GIF-files to their minimun size (that is RLE or no compression used)
		// The function takes a file-reference, $theFile, and saves it again through GD or ImageMagick in order to compress the file
		// GIF:
		// If $type is not set, the compression is done with ImageMagick (provided that $GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path_lzw"] is pointing to the path of a lzw-enabled version of 'convert') else with GD (should be RLE-enabled!)
		// If $type is set to either "IM" or "GD" the compression is done with ImageMagick and GD respectively
		// PNG:
		// No changes.

		// $theFile is expected to be a valid GIF-file!
		// The function returns a code for the operation.

		$gfxConf = $GLOBALS["TYPO3_CONF_VARS"]["GFX"];
		$returnCode="";
		if ($gfxConf["gif_compress"] && strtolower(substr($theFile,-4,4))==".gif")	{	// GIF...
			if (($type=="IM" || !$type) && $GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im"] && $GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path_lzw"])	{	// IM
				exec($GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path_lzw"].'convert "'.$theFile.'" "'.$theFile.'"');
				$returnCode="IM";
			} elseif (($type=="GD" || !$type) && $gfxConf["gdlib"] && !$gfxConf["gdlib_png"])	{	// GD
				$tempImage = imageCreateFromGif($theFile);
				imageGif($tempImage, $theFile);
				imageDestroy($tempImage);
				$returnCode="GD";
			}
		}
		return $returnCode;
	}

	/**
	 *  Converts a png file to gif
	 *
	 * This converts a png file to gif IF the FLAG $GLOBALS["TYPO3_CONF_VARS"]["FE"]["png_to_gif"] is set true.
	 *
	 * @param 	string $theFile	the filename with path
	 * @return 	string 	new filename
	 */
	function png_to_gif_by_imagemagick($theFile)	{
		if ($GLOBALS["TYPO3_CONF_VARS"]["FE"]["png_to_gif"]
			&& $GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im"]
			&& $GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path_lzw"]
			&& strtolower(substr($theFile,-4,4))==".png"
			&& @is_file($theFile))	{	// IM
				$newFile = substr($theFile,0,-4).".gif";
				exec($GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path_lzw"].'convert "'.$theFile.'" "'.$newFile.'"');
				$theFile = $newFile;
					// unlink old file?? May be bad idea bacause Typo3 would then recreate the file every time as Typo3 thinks the file is not generated because it's missing!! So do not unlink $theFile here!!
		}
		return $theFile;
	}

	/**
	 *  Returns filename of the png/gif version of the input file (which can be png or gif). 
	 *	If input file type does not match the wanted output type a conversion is made and temp-filename returned.
	 */
	function read_png_gif($theFile,$output_png=0)	{
		if ($GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im"] && @is_file($theFile))	{
			$ext = strtolower(substr($theFile,-4,4));
			if (
					((string)$ext==".png" && $output_png)	||
					((string)$ext==".gif" && !$output_png)
				)	{
				return $theFile;
			} else {
				$newFile = PATH_site."typo3temp/readPG_".md5($theFile."|".filemtime($theFile)).($output_png?".png":".gif");
				exec($GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path"].'convert "'.$theFile.'" "'.$newFile.'"');
#debug(array($GLOBALS["TYPO3_CONF_VARS"]["GFX"]["im_path"].'convert "'.$theFile.'" "'.$newFile.'"'));
				if (@is_file($newFile))	return $newFile;
			}
		}
	}















	## ************************
	## STRING FUNCTIONS
	## ************************

	/**
	 * Truncate string
	 *
	 * Returns a new string of max. $chars length.
	 * If the string is longer, it will be truncated and appended with "...".
	 *
	 * @param 	string $string 	string to truncate
	 * @param 	integer $chars 	must be an integer of at least 4
	 * @return 	string 	new string
	 */
	function fixed_lgd($string,$chars,$preStr="...")	{
		if ($chars >= 4)	{
			if(strlen($string)>$chars)  {
				return trim(substr($string, 0, $chars-3)).$preStr;
			}
		}
		return $string;
	}

	/**
	 * Truncate string
	 *
	 * Returns a new string of max. $chars length.
	 * If the string is longer, it will be truncated and prepended with "...".
	 *
	 * This works like fixed_lgd, but is truncated in the start of the string instead of the end
	 *
	 * @param 	string $string 	string to truncate
	 * @param 	integer $chars 	must be an integer of at least 4
	 * @return 	string 	new string
	 */
	function fixed_lgd_pre($string,$chars)	{
		return strrev(t3lib_div::fixed_lgd(strrev($string),$chars));
	}

	/**
	 * " " -> &nbsp;
	 *
	 * Replaces all chr(32)-characters (spaces) with "$nbsp;".
	 *
	 * @param 	string $string
	 * @return 	string 	new string
	 */
	function hardspace($string)		{
		return ereg_replace(" ", "&nbsp;", $string);
	}

	/**
	 * Inserts a "\n"-character (return-character) for each $interval character. Used for breaking up a base64 encoded string.
	 *
	 * @param  	string $inputstr
	 * @param 	char $interval 	character to replace with "\n"
	 * @return 	string 	new string
	 */
	function break_text($inputstr, $interval)	{
		$returncode = "\n";
		return chunk_split($inputstr,$interval,$returncode);
	}

	/**
	 * Breaks up the text for emails
	 */
	function breakTextForEmail($str,$implChar="\n",$charWidth=76)	{
		$lines = explode(chr(10),$str);
		$outArr=array();
		while(list(,$lStr)=each($lines))	{
			$outArr = array_merge($outArr,t3lib_div::breakLinesForEmail($lStr,$implChar,$charWidth));
		}
		return implode(chr(10),$outArr);
	}

	/**
	 * Breaks up the text for emails
	 */
	function breakLinesForEmail($str,$implChar="\n",$charWidth=76)	{
		$lines=array();
		$l=$charWidth;
		$p=0;
		while(strlen($str)>$p)	{
			$substr=substr($str,$p,$l);
			if (strlen($substr)==$l)	{
				$count = count(explode(" ",trim(strrev($substr))));
//				debug($parts);
//				debug($count);
//				if (count($parts)>1)	{	// OK...
				if ($count>1)	{	// OK...
					$parts = explode(" ",strrev($substr),2);
					$theLine = strrev($parts[1]);
				} else {
					$afterParts = explode(" ",substr($str,$l+$p),2);
					$theLine = $substr.$afterParts[0];
	//				debug($theLine);
				}
				if (!strlen($theLine))	{break; }	// Error, because this would keep us in an endless loop.
			} else {
				$theLine=$substr;
			}

			$lines[]=trim($theLine);
			$p+=strlen($theLine);
			if (!trim(substr($str,$p,$l)))	break;	// added...
		}
		return implode($implChar,$lines);
	}	
	
	/**
	 * Removes linebreaks
	 *
	 * Removes all "\n" characters (return-char) from $inputstr.
	 *
	 * @param  	string $inputstr
	 * @return 	string 	new string
	 */
	function unbreak_text($inputstr)	{
		return ereg_replace(chr(10),"",$inputstr);
	}

	/**
	 * Match IP number with list of numbers with wildcard
	 * $baseIP is the current remote IP address for instance and $list is a comma-list of IP-addresses to match with. *-wildcard allowed instead of number, plus leaving out parts in the IP number is accepted as wildcard (eg. 192.168.*.* equals 192.168)
	 */
	function cmpIP($baseIP, $list)	{
		$IPpartsReq = explode(".",$baseIP);
		if (count($IPpartsReq)==4)	{
			$values = t3lib_div::trimExplode(",",$list,1);
			reset($values);
			while(list(,$test)=each($values))	{
				$IPparts = explode(".",$test);
				$yes = 1;
				reset($IPparts);
				while(list($index,$val)=each($IPparts))	{
					$val = trim($val);
					if (strcmp($val,"*") && strcmp($IPpartsReq[$index],$val))	{
						$yes=0;
					}
				}
				if ($yes) return true;
			}
		}
	}

	/**
	 * Check for item in list
	 *
	 * Check if an item exists in a comma-separated list of items.
	 *
	 * @param 	string $in_list 	comma-separated list of items (string)
	 * @param 	string $item 	item to check for
	 * @return 	bool 		true if $item is in $in_list
	 */
	function inList($in_list,$item)	{
		return strstr(",".$in_list.",", ",".$item.",");
	}

	/**
	 * Removes an item from a comma-separated list of items.
	 *
	 * @param 	string $element  	element to remove
	 * @param 	string $list 	comma-separated list of items (string)
	 * @return 	string 	new comma-separated list of items
	 */
	function rmFromList($element,$list)	{
		$items = explode(",",$list);
		while(list($k,$v)=each($items))	{
			if ($v==$element)	{unset($items[$k]);}
		}
		return implode($items,",");
	}

	/**
	 * Forces the integer $theInt into the boundaries of $min and $max. If the $theInt is "false" then the $zeroValue is applied.
	 */
	function intInRange($theInt,$min,$max=2000000000,$zeroValue=0)	{
		// Returns $theInt as an integer in the integerspace from $min to $max
		$theInt = intval($theInt);
		if ($zeroValue && !$theInt)	{$theInt=$zeroValue;}	// If the input value is zero after being converted to integer, zeroValue may set another default value for it.
		if ($theInt<$min){$theInt=$min;}
		if ($theInt>$max){$theInt=$max;}
		return $theInt;
	}

	/**
	 * Returns the $integer if greater than zero, otherwise returns zero.
	 */
	function intval_positive($theInt)	{
		$theInt = intval($theInt);
		if ($theInt<0){$theInt=0;}
		return $theInt;
	}

	/**
	 * Returns an integer from a three part version number, eg "4.12.3" -> 4012003
	 */
	function int_from_ver($verNumberStr)	{
		$verParts = explode(".",$verNumberStr);
		return intval((int)$verParts[0].str_pad((int)$verParts[1],3,"0",STR_PAD_LEFT).str_pad((int)$verParts[2],3,"0",STR_PAD_LEFT));
	}

	/**
	 * Makes a positive integer hash out of the first 7 chars from the md5 hash of the input
	 */
	function md5int($str)	{
		return hexdec(substr(md5($str),0,7));
	}
	
	/**
	 * Takes a comma-separated list and removes all duplicates
	 */
	function uniqueList($in_list)	{
		// $in_list is a comma-separated list of values.
		// This function returns the list without any duplicates of values
		// space around values are trimmed
		$theTempArray = explode(",",$in_list);
		return implode(t3lib_div::uniqueArray($theTempArray),",");
	}

	/**
	 * This returns something like t3lib_div::split_fileref(not sure...)
	 */
	function extParseUrl($path)	{
		$res = parse_url($path);
		$res["fullpath"] = $path;
		ereg("(.*/)([^/]*)$",$res[path],$reg);
		$res["filepath"]=$reg[1];
		$res["filename"]=$reg[2];
		return $res;
	}

	/**
	 * Splits a reference to a file in 5 parts
	 * returns and array: [path], [file], [filebody], [fileext], [realFileext]
	 */
	function split_fileref($fileref)	{
		if (	ereg("(.*/)(.*)$",$fileref,$reg)	)	{
			$info["path"] = $reg[1];
			$info["file"] = $reg[2];
		} else {
			$info["path"] = "";
			$info["file"] = $fileref;
		}
		$reg="";
		if (	ereg("(.*)\.([^\.]*$)",$info[file],$reg)	)	{
			$info["filebody"] = $reg[1];
			$info["fileext"] = strtolower($reg[2]);
			$info["realFileext"] = $reg[2];
		} else {
			$info["filebody"] = $info[file];
			$info["fileext"] = "";
		}
		reset($info);	
		return $info;
	}
	
	/**
	 * Returns the directory part of a path without trailing slash
	 * If there is no dir-part, then an empty string is returned.
	 * Behaviour:
	 *
	 *	"/dir1/dir2/script.php" => "/dir1/dir2"
	 *	"/dir1/" => "/dir1"
	 *	"dir1/script.php" => "dir1"
	 *	"d/script.php" => "d"
	 *	"/script.php" => ""
	 *	"" => ""
	 */
	function dirname($path)	{
		$p=t3lib_div::revExplode("/",$path,2);
		return count($p)==2?$p[0]:"";
	}

	/**
	 * Modifies a HTML Hex color by adding/subtracting $R,$G and $B integers
	 */
	function modifyHTMLColor($color,$R,$G,$B)	{
		// This takes a hex-color (# included!) and adds $R, $G and $B to the HTML-color (format: #xxxxxx) and returns the new color
		$nR = t3lib_div::intInRange(hexdec(substr($color,1,2))+$R,0,255);
		$nG = t3lib_div::intInRange(hexdec(substr($color,3,2))+$G,0,255);
		$nB = t3lib_div::intInRange(hexdec(substr($color,5,2))+$B,0,255);
		return "#".
			substr("0".dechex($nR),-2).
			substr("0".dechex($nG),-2).
			substr("0".dechex($nB),-2);
	}

	/**
	 * Modifies a HTML Hex color by adding/subtracting $all integer from all R/G/B channels
	 */
	function modifyHTMLColorAll($color,$all)	{
		return t3lib_div::modifyHTMLColor($color,$all,$all,$all);
	}

	/**
	 * Removes comma in the end of string
	 */
	function rm_endcomma($string)	{
		return ereg_replace(",$","",$string);
	}

	/**
	 * strtoupper which converts danish (and other characters) characters as well
	 * (Depreciated, use PHP function with locale settings instead)
	 */
	function danish_strtoupper($string)	{
		$value = strtoupper($string);
		return strtr($value, "", "");
	}

	/**
	 * Returns the first 10 positions of the MD5-hash		(changed from 6 to 10 recently)
	 */
	function shortMD5($input, $len=10)	{
		return substr(md5($input),0,$len);
	}

	/**
	 * returns true if string is an integer
	 */
	function testInt($var)	{
		return !strcmp($var,intval($var));
	}

	/**
	 * Returns true if the first part of $str matches the string $partStr
	 */
	function isFirstPartOfStr($str,$partStr)	{
		// Returns true, if the first part of a $str equals $partStr and $partStr is not ""
		$psLen = strlen($partStr);
		if ($psLen)	{
			return substr($str,0,$psLen)==(string)$partStr;
		} else return false;
	}

	/**
	 * Formats the input integer $sizeInBytes as bytes/kilobytes/megabytes (-/K/M)
	 */
	function formatSize($sizeInBytes)	{
		// Formats a number to Mb or Kb or just bytes
		if ($sizeInBytes>900)	{
			if ($sizeInBytes>900000)	{	// MB
				$val = $sizeInBytes/(1024*1024);
				return number_format($val, (($val<20)?1:0), ".", "")." M";
			} else {	// KB
				$val = $sizeInBytes/(1024);
				return number_format($val, (($val<20)?1:0), ".", "")." K";
			}
		} else {	// Bytes
			return $sizeInBytes."&nbsp;&nbsp;";
		}
	}

	/** 
	 * Returns microtime input to milliseconds
	 */
	function convertMicrotime($microtime)	{
		$parts = explode(" ",$microtime);
		return round(($parts[0]+$parts[1])*1000);
	}

	/**
	 * This splits a string by the chars in $operators (typical /+-*) and returns an array with them in
	 */
	function splitCalc($string,$operators)	{
		$res = Array();
		$sign="+";
		while($string)	{
			$valueLen=strcspn($string,$operators);
			$value=substr($string,0,$valueLen);
			$res[] = Array($sign,trim($value));
			$sign=substr($string,$valueLen,1);
			$string=substr($string,$valueLen+1);
		}
		reset($res);
		return $res;
	}

	/**
	 * Calculates the input by +,-,*,/,%,^ with priority to + and -
	 */
	function calcPriority ($string)	{
//		$string = "-5*-3446.234*-7*8 + 9 ^ - 3 +-20%-3 /-2 / 3";
		$string=ereg_replace("[[:space:]]*","",$string);	// removing all whitespace
		$string="+".$string;	// Ensuring an operator for the first entrance
		$qm="\*\/\+-^%";
		$regex = "([$qm])([$qm]?[0-9\.]*)";
			// split the expression here:
		preg_match_all("/".$regex."/",$string,$reg);
//		debug($reg);	// This is the numbers and their operators
		
		reset($reg[2]);
		$number=0;
		$Msign="+";
		$err="";	
		$buffer=doubleval(current($reg[2]));
		next($reg[2]);	// Advance pointer
		while(list($k,$v)=each($reg[2]))	{
			$v=doubleval($v);
			$sign = $reg[1][$k];
			if ($sign=="+" || $sign=="-")	{
				$number = $Msign=="-" ? $number-=$buffer : $number+=$buffer;
				$Msign = $sign;
				$buffer=$v;
			} else {
				if ($sign=="/")	{if ($v) $buffer/=$v; else $err="dividing by zero";}
				if ($sign=="%")	{if ($v) $buffer%=$v; else $err="dividing by zero";}
				if ($sign=="*")	{$buffer*=$v;}
				if ($sign=="^")	{$buffer=pow($buffer,$v);}
			}
		}
		$number = $Msign=="-" ? $number-=$buffer : $number+=$buffer;
		return $err ? "ERROR: ".$err : $number;
	}

	/**
	 * Calculates the input with parenthesis levels
	 */
	function calcParenthesis ($string)	{
		$securC=100;
		do {
			$valueLenO=strcspn($string,"(");
			$valueLenC=strcspn($string,")");
			if ($valueLenC==strlen($string) || $valueLenC < $valueLenO)	{
				$value = t3lib_div::calcPriority(substr($string,0,$valueLenC));
				$string = $value.substr($string,$valueLenC+1);
				return $string;
			} else {
				$string = substr($string,0,$valueLenO).t3lib_div::calcParenthesis(substr($string,$valueLenO+1));
			}
				// Security:
			$securC--;
			if ($securC<=0)	break;
		} while($valueLenO<strlen($string));
		return $string;
	}

	/**
	 * Inverse version of htmlspecialchars()
	 */
	function htmlspecialchars_decode($value)	{
		$value = str_replace("&gt;",">",$value);
		$value = str_replace("&lt;","<",$value);
		$value = str_replace("&quot;",'"',$value);
		$value = str_replace("&amp;","&",$value);
		return $value;
	}

	/**
	 * This function is used to escape any ' -characters when transferring text to JavaScript!
	 */
	function slashJS($string,$extended=0,$char="'")	{
		if ($extended)	{$string = str_replace ("\\", "\\\\", $string);}
		return str_replace ($char, "\\".$char, $string);
	}	

	/**
	 * version of raw_url_encode where all spaces (%20) are re-converted to space-characters.
	 * Usefull when passing text to JavaScript where you simply url-encode it to get around problems with syntax-errors, linebreaks etc.
	 */
	function rawUrlEncodeJS($str)	{
		return str_replace("%20"," ",rawurlencode($str));
	}

	/**
	 * Returns true if the $email address is valid
	 */
	function validEmail($email)	{
		if (strstr($email," "))	 return 0;
		return ereg("^[A-Za-z0-9\._-]*[@][A-Za-z0-9\._-]*[\.].[A-Za-z0-9]*$",$email) ? 1 : 0;
	}

	/**
	 * All content outputted in a textarea form should be passed through this function
	 * Not only is the content htmlspecialchar'ed on output but there is also a single newline added in the top. The newline is necessary because browsers will ignore the first newline after <textarea> if that is the first character. Therefore better set it!
	 */
	function formatForTextarea($content)	{
		return chr(10).htmlspecialchars($content);
	}


	
	
	
	
	
	
	



	## ************************
	## ARRAY FUNCTIONS
	## ************************

	/**
	 * Check if an item exists in an array
	 *
	 * Please note that the order of parameters is reverse compared to the php4-function in_array()!!!
	 *
	 * @param 	array  $in_array		one-dimensional array of items
	 * @param  	string $item 	item to check for
	 * @return 	bool 		true if $item is in the one-dimensional array $in_array
	 */
	function inArray($in_array,$item)	{
		if (is_array($in_array))	{
			while (list(,$val)=each($in_array))	{
				if (!is_array($val) && !strcmp($val,$item)) return true;
			}
		}
	}

	/**
	 * Explodes a $string delimited by $delim and passes each item in the array through intval().
	 * Corresponds to explode(), but with conversion to integers for all values.
	 */
	function intExplode($delim, $string)	{
		// This explodes a comma-list into an array where the values are parsed through intval();
		$temp = explode($delim,$string);
		while(list($key,$val)=each($temp))	{
			$temp[$key]=intval($val);
		}
		reset($temp);
		return $temp;
	}

	/**
	 * Reverse explode which explodes the string counting from behind.
	 * Thus t3lib_div::revExplode(":","my:words:here",2) will return array("my:words","here")
	 */
	function revExplode($delim, $string, $count=0)	{
		$temp = explode($delim,strrev($string),$count);
		while(list($key,$val)=each($temp))	{
			$temp[$key]=strrev($val);
		}
		$temp=array_reverse($temp);
		reset($temp);
		return $temp;
	}

	/**
	 * Explodes a string and trims all values for whitespace in the ends. 
	 * If $onlyNonEmptyValues is set, then all blank ("") values are removed.
	 */
	function trimExplode($delim, $string, $onlyNonEmptyValues=0)	{
		// This explodes a comma-list into an array where the values are parsed through trim();
		$temp = explode($delim,$string);
		$newtemp=array();
		while(list($key,$val)=each($temp))	{
			if (!$onlyNonEmptyValues || strcmp("",trim($val)))	{
				$newtemp[]=trim($val);
			}
		}
		reset($newtemp);
		return $newtemp;
	}

	/**
	 * Takes a one-dimensional array and returns an array where the values are unique
	 * The keys in the array are substituted with some md5-hashes
	 * If the value is trim(empty), the value is ignored.
	 * Values are trimmed
	 * (Depreciated, use PHP function instead)
	 */
	function uniqueArray($valueArray)	{
		$array_out=array();
		if (is_array($valueArray))	{
			while (list($key,$val)=each($valueArray)) {
				$val=trim($val);
				if ((string)$val!="")	{
					$array_out[md5($val)]=$val;
				}
			}
		}
		reset($array_out);
		return $array_out;
	}

	/**
	 * Removes the value $cmpValue from the $array if found there. Returns the modified array
	 */
	function removeArrayEntryByValue($array,$cmpValue)	{
		if (is_array($array))	{
			reset($array);
			while(list($k,$v)=each($array))	{
				if (is_array($v))	{
					$array[$k] = t3lib_div::removeArrayEntryByValue($v,$cmpValue);
				} else {
					if (!strcmp($v,$cmpValue))	{
						unset($array[$k]);
					}
				}
			}
		}
		reset($array);
		return $array;
	}

	/** 
	 * implodes a multidim-array into GET-parameters (eg. &param[key][key2]=value2&param[key][key3]=value3)
	 */
	function implodeArrayForUrl($name,$theArray,$str="",$skipBlank=0)	{	
		if (is_array($theArray))	{
			reset($theArray);
			while(list($Akey,$AVal)=each($theArray))	{
				$thisKeyName = $name ? $name."[".$Akey."]" : $Akey;
				if (is_array($AVal))	{
					$str=t3lib_div::implodeArrayForUrl($thisKeyName,$AVal,$str,$skipBlank);
				} else {
					if (!$skipBlank || strcmp($AVal,""))	$str.="&".$thisKeyName."=".rawurlencode(stripslashes($AVal));	// strips slashes because HTTP_POST_VARS / GET_VARS input is with slashes...
				}
			}
		}
		return $str;
	}

	/**
	 * Returns an array with selected keys from incoming data.
	 * (Better read source code if you want to find out...)
	 */
	function compileSelectedGetVarsFromArray($varList,$getArray,$GPvarAlt=1)	{
		$keys = t3lib_div::trimExplode(",",$varList,1);
		$outArr=array();
		while(list(,$v)=each($keys))	{
			if (isset($getArray[$v]))	{
				$outArr[$v]=$getArray[$v];
			} elseif ($GPvarAlt) {
				$outArr[$v]=t3lib_div::GPvar($v);
			}
		}
		return $outArr;
	}	

	/**
	 * AddSlash array
	 *
	 * This function traverses a multidimentional array and adds slashes to the values.
	 * NOTE that the input array is and argument by reference.!!
	 * Twin-function to stripSlashesOnArray
	 */
	function addSlashesOnArray(&$theArray)	{
		if (is_array($theArray))	{
			reset($theArray);
			while(list($Akey,$AVal)=each($theArray))	{
				if (is_array($AVal))	{
					t3lib_div::addSlashesOnArray($theArray[$Akey]);
				} else {
					$theArray[$Akey] = addslashes($AVal);
				}
			}
			reset($theArray);
		}
	}

	/**
	 * StripSlash array
	 *
	 * This function traverses a multidimentional array and strips slashes to the values.
	 * NOTE that the input array is and argument by reference.!!
	 * Twin-function to addSlashesOnArray
	 */
	function stripSlashesOnArray(&$theArray)	{
		if (is_array($theArray))	{
			reset($theArray);
			while(list($Akey,$AVal)=each($theArray))	{
				if (is_array($AVal))	{
					t3lib_div::stripSlashesOnArray($theArray[$Akey]);
				} else {
					$theArray[$Akey] = stripslashes($AVal);
				}
			}
			reset($theArray);
		}
	}

	/**
	 * Either slashes ($cmd=add) or strips ($cmd=strip) array $arr depending on $cmd
	 */
	function slashArray($arr,$cmd)	{
		if ($cmd=="strip")	t3lib_div::stripSlashesOnArray($arr);
		if ($cmd=="add")	t3lib_div::addSlashesOnArray($arr);
		return $arr;
	}

	/**
	 *  Merges two arrays recursively, overruling the values of the first array
	 *  in case of identical keys, ie. keeping the values of the second.
	 */
	function array_merge_recursive_overrule ($arr0,$arr1,$notAddKeys=0) {
		reset($arr1);
		while(list($key,$val) = each($arr1)) {
			if(is_array($arr0[$key])) {
				if (is_array($arr1[$key]))	{
					$arr0[$key] = t3lib_div::array_merge_recursive_overrule($arr0[$key],$arr1[$key],$notAddKeys);
				}
			} else {
				if ($notAddKeys) {
					if (isset($arr0[$key])) {
						$arr0[$key] = $val;
					}
				} else {
					$arr0[$key] = $val;
				}
			}
		}
		reset($arr0);
		return $arr0;
	}
	
	/**
	 * An array_merge function where the keys are NOT renumbered as they happen to be with the real php-array_merge function
	 */
	function array_merge($arr1,$arr2)	{
		return $arr2+$arr1;
	}
	
	/**
	 * Takes a row and returns a CSV string of the values with $delim (default is ,) and $quote (default is ") as separator chars.
	 */
	function csvValues($row,$delim=",",$quote='"')	{
		reset($row);
		$out=array();
		while(list(,$value)=each($row))	{
			list($valPart) = explode(chr(10),$value);
			$valPart = trim($valPart);
			$out[]=str_replace($quote,$quote.$quote,$valPart);
		}
		$str = $quote.implode($quote.$delim.$quote,$out).$quote;
		return $str;
	}


	













	## ***********************
	## HTML PROCESSING
	## ***********************

	/**
	 * $tag is either a whole tag (eg "<TAG OPTION ATTRIB=VALUE>") or the parameterlist (ex " OPTION ATTRIB=VALUE>")
	 * Returns an array with all attributes as keys. Attributes are only lowercase a-z
	 * If a attribute is empty (I call it "an option"), then the value for the key is empty. You can check if it existed with isset()
	 */
	function get_tag_attributes($tag)	{
		$components = t3lib_div::split_tag_attributes($tag);
		$name = "";	 // attribute name is stored here
		$valuemode = "";
		if (is_array($components))	{
			while (list($key,$val) = each ($components))	{
				if ($val != "=")	{	// Only if $name is set (if there is an attribute, that waits for a value), that valuemode is enabled. This ensures that the attribute is assigned it's value
					if ($valuemode)	{
						if ($name)	{
							$attributes[$name] = $val;
							$name = "";
						}
					} else {
						if ($key = strtolower(ereg_replace("[^a-zA-Z0-9]","",$val)))	{
							$attributes[$key] = "";
							$name = $key;
						}
					}
					$valuemode = "";
				} else {
					$valuemode = "on";
				}
			}
			if (is_array($attributes))	reset($attributes);
			return $attributes;
		}
	}

	/**
	 * Returns an array with the "components" from an attribute list from an HTML tag. The result is normally analyzed by get_tag_attributes
	 * Removes tag-name if found
	 */
	function split_tag_attributes($tag)	{
		$tag_tmp = trim(eregi_replace ("^[ ]*<[^ ]*","",$tag));
			// Removes any > in the end of the string
		$tag_tmp = trim(eregi_replace (">$","",$tag_tmp));

		while (strcmp($tag_tmp,""))	{	// Compared with empty string instead , 030102
			$firstChar=substr($tag_tmp,0,1);
			if (!strcmp($firstChar,'"') || !strcmp($firstChar,"'"))	{
				$reg=explode($firstChar,$tag_tmp,3);
				$value[]=$reg[1];
				$tag_tmp=trim($reg[2]);
			} elseif (!strcmp($firstChar,"=")) {
				$value[] = "=";
				$tag_tmp = trim(substr($tag_tmp,1));		// Removes = chars.
			} else {
				// There are "" around the value. We look for the next " " or ">"
				$reg = split("[[:space:]=]",$tag_tmp,2);
				$value[] = trim($reg[0]);
				$tag_tmp = trim(substr($tag_tmp,strlen($reg[0]),1).$reg[1]);
			}
		}
		if (is_array($value))	reset($value);
		return $value;
	}

	/**
	 * Implodes attributes in the array $arr for an attribute list in eg. and HTML tag (with quotes)
	 */
	function implodeParams($arr)	{
		if (is_array($arr))	{
			reset($arr);
			$list = array();
			while(list($p,$v)=each($arr))	{
				if (strcmp($v,""))	{$list[]=$p.'="'.$v.'"';}
			}
			return implode(" ",$list);
		}
	}

	/**
	 * Finds all (unique) image-references in $html_code's IMG-tags
	 * Does not cover JavaScript, background images and images in form-buttons
	 */
	function imagefinder($html_code)	{
		// returns a two-dim array with all the pictures:
		// $images[$i][filename][fullpath]
		// $images[$i][filename][name]
		// $images[$i][filename][path]
		// $images[$i][width]
		// $images[$i][height]
		// $images[$i][alt]

		$codepieces = split(sql_regcase("<img "), $html_code, 1000000 );
		$pieces = count($codepieces);
		$image_fullpath_list = "";

		for($i=1; $i < $pieces; $i++)	{
			$dummy = eregi("[^>]*", $codepieces[$i], $reg);
			$image = $reg[0];
			$attributes = t3lib_div::get_tag_attributes($image);
			$img_pathname = split_path_and_name ($attributes[src]);
			if (!strstr($image_fullpath_list,"|".$img_pathname[fullpath]."|") )
			{
				$image_fullpath_list.="|".$img_pathname[fullpath]."|";
				$images[$i][filename]=$img_pathname;
				$images[$i][width]=$attributes[width];
				$images[$i][height]=$attributes[height];
				$images[$i][alt]=$attributes[alt];
			}
		}
		if (is_array($images))	reset($images);		
		return $images;
	}

	/**
	 * Finds all links in $html_code
	 */
	function linkfinder($html_code)	{
		$codepieces = split(sql_regcase("<a "), $html_code, 1000000 );
		$pieces = count($codepieces);
		$image_fullpath_list = "";

		for($i=1; $i < $pieces; $i++)	{
			$dummy = eregi("[^>]*", $codepieces[$i], $reg);
			$link = $reg[0];
			$attributes = t3lib_div::get_tag_attributes($link);
			$link_pathname = split_path_and_name ($attributes[href]);
			if (!strstr($link_fullpath_list,"|".$link_pathname[fullpath]."|") )
			{
				$link_fullpath_list.="|".$link_pathname[fullpath]."|";
				$links[$i][filename]=$link_pathname;
			}
		}
		if (is_array($links))	reset($links);
		return $links;
	}



	
	
	
	
	
	
	
	
	
	
	## ********************
	## FILES FUNCTIONS
	## ********************
	
	

	/**
	 * Reads the file or url $url and returns the content
	 */
	function getURL($url)	{
		$content = "";
		
			// (Proxy support implemented by Arco <arco@appeltaart.mine.nu>)
		if((substr($url,0,7)=="http://") && ($GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlUse"]=='1'))	{
			//external URL without error checking.
			$ch = curl_init();
			curl_setopt ($ch,CURLOPT_URL, $url);
			curl_setopt ($ch,CURLOPT_HEADER, 0);
			curl_setopt ($ch,CURLOPT_RETURNTRANSFER, 1);
			
			if ($GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlProxyServer"]) {
				curl_setopt ($ch, CURLOPT_PROXY, $GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlProxyServer"]);
				
				// I don't know if it will be needed
				if ($GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlProxyTunnel"]) {
					curl_setopt ($ch, CURLOPT_HTTPPROXYTUNNEL, $GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlProxyTunnel"] );
				}
				if ($GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlProxyUserPass"]) {
					curl_setopt ($ch, CURLOPT_PROXYUSERPWD, $GLOBALS["TYPO3_CONF_VARS"]["SYS"]["curlProxyUserPass"] );
				}
			}
			$content=curl_exec ($ch);
			curl_close ($ch);           
			return $content;   
		} elseif($fd = fopen($url,"r"))    {
			while (!feof($fd))	{
				$content.=fread($fd, 5000);
			}
			fclose($fd);
			return $content;
		}
	}

	/**
	 * Reads the file or url $url and returns the content. Any HTML is stripped
	 * DEPRECIATED (and probably not used anywhere anyways. Just use getURL() (above) and strip_tags afterwards!
	 */
	function getStrippedURL($url)	{
		if($fd = fopen($url,"r"))	{
			$content = "";
			while (!feof($fd))	{
				$content.=trim(fgetss($fd,50000));
			}
			fclose($fd);
			return $content;
		}
	}

	/**
	 * This functions tries to open a url and decide the mime-type of that url
	 */
	function getMimeType($url)	{
		$pathInfo = parse_url($url);
		$getAdr = ($pathInfo[query])?$pathInfo[path]."?".$pathInfo[query]:$pathInfo[path];
		$fp = fsockopen($pathInfo[host], 80, $errno, $errstr);
		if(!$fp) {
			return false;
		} else {
	        fputs($fp,"GET ".$getAdr." HTTP/1.0\n\n");
	        while(!feof($fp)) {
				$thePortion= fgets($fp,128);
				if (eregi("(^Content-Type: )(.*)",trim($thePortion), $reg))	{
					$res = trim($reg[2]);
					break;
				}
	        }
	        fclose($fp);
		}
		return $res;
	}

	/**
	 * Writes $content to the file $file
	 */
	function writeFile($file,$content)	{
		if($fd = fopen($file,"w"))	{
			fwrite( $fd, $content);
			fclose( $fd );
			if (@is_file($file) && TYPO3_OS!="WIN")	{@chmod ($file, 0644);}
			return true;
		}
	}

	/**
	 * Returns an array with the names of folders in a specific path
	 * Will return "error" (string) if there were an error with reading directory content.
	 */
	function get_dirs($path)	{
		if ($path)	{
			$d = @dir($path);
			if (is_object($d))	{
				while($entry=$d->read()) {
					if (@is_dir($path."/".$entry) && $entry!= ".." && $entry!= ".")	{
					    $filearray[]=$entry;
					}
				}
				$d->close();
			} else return "error";
			return $filearray;
		}
	}

	/**
	 * Returns an array with the names of files in a specific path
	 * $path: Is the path to the file
	 * $extensionList is the comma list of extensions to read only (blank = all)
	 * $prependPath (boolean) : If set, then the path is prepended the filenames. Otherwise only the filenames are returned in the array
	 * $order is sorting: 1= sort alphabetically, "mtime" = sort by modification time.
	 */
	function getFilesInDir($path,$extensionList="",$prependPath=0,$order="")	{
		$filearray=array();
		$sortarray=array();
		if ($path)	{
			$path = ereg_replace("/$","",$path);
			$extensionList = strtolower($extensionList);
			$d = @dir($path);
			if (is_object($d))	{
				while($entry=$d->read()) {
					if (@is_file($path."/".$entry))	{	
						$fI = pathinfo($entry);
						$key = md5($path."/".$entry);
						if (!$extensionList || t3lib_div::inList($extensionList,strtolower($fI["extension"])))	{
						    $filearray[$key]=($prependPath?$path."/":"").$entry;
							if ($order=="mtime") {$sortarray[$key]=filemtime($path."/".$entry);}
								elseif ($order)	{$sortarray[$key]=$entry;}
						}
					}
				}
				$d->close();
			} else return "error";
		}
		if ($order) {
			asort($sortarray);
			reset($sortarray);
			$newArr=array();
			while(list($k,$v)=each($sortarray))	{
				$newArr[$k]=$filearray[$k];
			}
			$filearray=$newArr;
		}
		reset($filearray);
		return $filearray;
	}

	/**
	 * Moves $source file to $destination if uploaded, otherwise try to make a copy
	 */
	function upload_copy_move($source,$destination)	{
		if (is_uploaded_file($source))	{
			move_uploaded_file ($source, $destination);
		} else @copy($source,$destination);
		if (@is_file($destination) && TYPO3_OS!="WIN")	{@chmod ($destination, 0644);}
	}

	/**
	 * Fixes a path for windows-backslashes and reduces double-slashes to single slashes
	 */
	function fixWindowsFilePath($theFile)	{
		return str_replace('//','/', str_replace('\\','/', $theFile));
	}

	/**
	 * Prefixes a URL used with "header-location" with "http://..." depending on whether it has it already.
	 * - If already having a scheme, nothing is prepended
	 * - If having REQUEST_URI slash "/", then prefixing "http://[host]" (relative to host)
	 * - Otherwise prefixed with TYPO3_REQUEST_DIR (relative to current dir / TYPO3_REQUEST_DIR)
	 */
	function locationHeaderUrl($path)	{
		$uI = parse_url($path);
		if (substr($path,0,1)=="/")	{ // relative to HOST
			$path = t3lib_div::getIndpEnv("TYPO3_REQUEST_HOST").$path;
		} elseif (!$uI["scheme"])	{ // No scheme either
			$path = t3lib_div::getIndpEnv("TYPO3_REQUEST_DIR").$path;
		} 
		return $path;
	}	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	## *************************
	## DEBUG helper FUNCTIONS
	## *************************

	/**
	 * Returns a string with a list of ascii-values for all whitespace in the beginning of $string
	 */
	function debug_white_space($string)	{
		if(eregi("^[ ".chr(13).chr(10)."]*",$string,$reg))	{
			return debug_print_ordvalue($reg[0],strlen($reg[0]));
		}
	}

	/**
	 * Returns a string with a list of ascii-values for the first $characters characters in $string
	 */
	function debug_ordvalue($string,$characters=100)	{
		if(strlen($string) < $characters)
			$characters = strlen($string);
		for ($i=0; $i<$characters; $i++)	{
			$valuestring .= " ".ord(substr($string,$i,1));
		}
		return ereg_replace("^ ","",$valuestring);
	}

	/**
	 * Prints $string between two <HR> tags
	 */
	function debug_code($string)	{
		echo "<hr>";
		echo nl2br(HTMLSpecialChars($string));
		echo "<hr>";
	}

	/**
	 * Prints the difference in milliseconds between the current time and $starttime (milliseconds)
	 * This is useful for measuring the execution speed by first loading the time in t3lib_div::milliseconds($pretime=t3lib_div::milliseconds()) and after a function-call pass $pretime to t3lib_div::parsetime()
	 */
	function parsetime($starttime)	{
		echo '<font face="Verdana" size="1" color="Black"><b>Parsetime: '.(t3lib_div::milliseconds()-$starttime).' ms</b></font>';
	}

	/**
	 * Returns HTML-code, which is a visual representation of a multidimensional array
	 * use t3lib_div::print_array() in order to print an array
	 * Returns false if $array_in is not an array
	 */
	function view_array($array_in)	{
		if (is_array($array_in))	{
			$result='<table border=1 cellpadding=1 cellspacing=0 bgcolor=white>';
			if (!count($array_in))	{$result.= '<tr><td><font face="Verdana,Arial" size="1"><b>'.HTMLSpecialChars("EMPTY!").'</b></font></td></tr>';}
			while (list($key,$val)=each($array_in))	{
				$result.= '<tr><td><font face="Verdana,Arial" size="1">'.HTMLSpecialChars($key).'</font></td><td>';
				if (is_array($array_in[$key]))	{
					$result.=t3lib_div::view_array($array_in[$key]);
				} else
					$result.= '<font face="Verdana,Arial" size="1" color=red>'.nl2br(HTMLSpecialChars($val)).'<BR></font>';
				$result.= '</td></tr>';
			}
			$result.= '</table>';
		} else	{
			$result  = false;
		}
		return $result;
	}

	/**
	 * Prints an array
	 */
	function print_array($array_in)	{
		echo t3lib_div::view_array($array_in);
	}

	/**
	 * makes debug output
	 *
	 * Prints $var in bold between two vertical lines
	 * If not $var the word "debug" is printed
	 * If $var is an array, the array is printed by t3lib_div::print_array()
	 */
	function debug($var="",$br=0)	{
		// Prints $var in bold between two vertical lines
		// If not $var the word "debug" is printed
		// If $var is an array, the array is printed by t3lib_div::print_array()

		if ($br<0) {for($a=0;$a<abs(intval($br));$a++){echo "<BR>";}}
		if (is_array($var))	{
			t3lib_div::print_array($var);
		} elseif ((string)$var!="")
			echo "<B>|".HTMLSpecialChars((string)$var)."|</B>";
		else
			echo "<B>| debug |</B>";

		if ($br>0) {for($a=0;$a<intval($br);$a++){echo "<BR>";}}
	}

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

	
	
	
	
	
	
	
	
	
	
	





	## **************************
	## SYSTEM INFORMATION	
	## **************************
	
	/**
	 * Returns the HOST+DIR-PATH of the current script (The URL, but without "http://" and without script-filename)
	 */
	function getThisUrl()	{
		$p=parse_url(t3lib_div::getIndpEnv("TYPO3_REQUEST_SCRIPT"));		// Url of this script
		$dir=t3lib_div::dirname($p["path"])."/";	// Strip file
		$url = str_replace("//","/",$p["host"].($p["port"]?":".$p["port"]:"").$dir);
		return $url;
	}

	/** 
	 * Returns the link-url to the current script. 
	 * In $getParams you can set associative keys corresponding to the get-vars you wish to add to the url. If you set them empty, they will remove existing get-vars from the current url.
	 */
	function linkThisScript($getParams=array())	{	
		$parts = t3lib_div::getIndpEnv("SCRIPT_NAME");
		$params = $GLOBALS["HTTP_GET_VARS"];
		reset($getParams);
		while(list($k,$v)=each($getParams))	{
			if (strcmp($v,""))	{
				$params[$k]=$v;
			} else unset($params[$k]);
		}
		$pString = t3lib_div::implodeArrayForUrl("",$params);
		return $pString ? $parts."?".ereg_replace("^&","",$pString) : $parts;
	}

	/**
	 * Takes a full URL, $url, possibly with a querystring and overlays the $getParams arrays values onto the quirystring, packs it all together and returns the URL again.
	 * So basically it adds the parameters in $getParams to an existing URL, $url
	 */
	function linkThisUrl($url,$getParams=array())	{
		$parts = parse_url($url);
		if ($parts["query"])	{
			parse_str($parts["query"],$getP);
		} else $getP=array();
		$getP=t3lib_div::array_merge_recursive_overrule($getP,$getParams);
		$uP = explode("?",$url);
		$outurl = $uP[0]."?".t3lib_div::implodeArrayForUrl("",$getP);
		return $outurl;
	}

	/**
	 * Abstraction method which returns System Environment Variables regardless of server OS, CGI/MODULE version etc. Basically this is SERVER variables for most of them.
	 * This should be used instead of getEnv() and HTTP_SERVER_VARS/ENV_VARS to get reliable values for all situations.
	 */
	function getIndpEnv($getEnvName)	{
		global $HTTP_SERVER_VARS;
		/* 
			Conventions:
			output from parse_url():
			URL:	http://username:password@192.168.1.4:8080/typo3/32/temp/phpcheck/index.php/arg1/arg2/arg3/?arg1,arg2,arg3&p1=parameter1&p2[key]=value#link1
			    [scheme] => 'http'
			    [user] => 'username'
			    [pass] => 'password'
			    [host] => '192.168.1.4'
				[port] => '8080'
			    [path] => '/typo3/32/temp/phpcheck/index.php/arg1/arg2/arg3/'
			    [query] => 'arg1,arg2,arg3&p1=parameter1&p2[key]=value'
			    [fragment] => 'link1'
				
				Further definition: [path_script] = '/typo3/32/temp/phpcheck/index.php'
									[path_dir] = '/typo3/32/temp/phpcheck/'
									[path_info] = '/arg1/arg2/arg3/'
									[path] = [path_script/path_dir][path_info]
										

			Keys supported:

			URI______:
				REQUEST_URI		=	[path]?[query]		= /typo3/32/temp/phpcheck/index.php/arg1/arg2/arg3/?arg1,arg2,arg3&p1=parameter1&p2[key]=value
				HTTP_HOST		=	[host][:[port]]		= 192.168.1.4:8080
				SCRIPT_NAME		=	[path_script]++		= /typo3/32/temp/phpcheck/index.php		// NOTICE THAT SCRIPT_NAME will return the php-script name ALSO. [path_script] may not do that (eg. "/somedir/" may result in SCRIPT_NAME "/somedir/index.php")!
				PATH_INFO		=	[path_info]			= /arg1/arg2/arg3/
				QUERY_STRING	=	[query]				= arg1,arg2,arg3&p1=parameter1&p2[key]=value
				HTTP_REFERER	=	[scheme]://[host][:[port]][path]	= http://192.168.1.4:8080/typo3/32/temp/phpcheck/index.php/arg1/arg2/arg3/?arg1,arg2,arg3&p1=parameter1&p2[key]=value
										(Notice: NO username/password + NO fragment)

			CLIENT____:
				REMOTE_ADDR		=	(client IP)
				REMOTE_HOST		=	(client host)
				HTTP_USER_AGENT	=	(client user agent)
				HTTP_ACCEPT_LANGUAGE	= (client accept language)
	
			SERVER____:
				SCRIPT_FILENAME	=	Absolute filename of script		(Differs between windows/unix). On windows "C:\\blabla\\blabl\\" will be converted to "C:/blabla/blabl/"
				
			Special extras:
				TYPO3_HOST_ONLY	=		[host]			= 192.168.1.4
				TYPO3_PORT		=		[port]			= 8080 (blank if 80, taken from host value)
				TYPO3_REQUEST_HOST = 	[scheme]://[host][:[port]]
				TYPO3_REQUEST_URL =		[scheme]://[host][:[port]][path]?[query]	(sheme will by default be "http" until we can detect if it's https - 
				TYPO3_REQUEST_SCRIPT =  [scheme]://[host][:[port]][path_script]
				TYPO3_REQUEST_DIR =		[scheme]://[host][:[port]][path_dir]
				TYPO3_SITE_URL = 		[scheme]://[host][:[port]][path_dir] of the Typo3 website 
				TYPO3_DOCUMENT_ROOT	=	Absolute path of root of documents:	TYPO3_DOCUMENT_ROOT.SCRIPT_NAME = SCRIPT_FILENAME (typically)
		
			Notice: [fragment] is apparently NEVER available to the script!
			
			
			Testing suggestions:
			- Output all the values.
			- In the script, make a link to the script it self, maybe add some parameters and click the link a few times so HTTP_REFERER is seen
			- ALSO TRY the script from the ROOT of a site (like "http://www.mytest.com/" and not "http://www.mytest.com/test/" !!)
		
		*/
		
#		if ($getEnvName=="HTTP_REFERER")	return "";
		switch((string)$getEnvName)	{
			case "SCRIPT_NAME":
				return php_sapi_name()=="cgi" ? $HTTP_SERVER_VARS["PATH_INFO"] : $HTTP_SERVER_VARS["SCRIPT_NAME"];
			break;
			case "SCRIPT_FILENAME":
				return str_replace('//','/', str_replace('\\','/', php_sapi_name()=="cgi"||php_sapi_name()=="isapi" ? $HTTP_SERVER_VARS["PATH_TRANSLATED"]:$HTTP_SERVER_VARS["SCRIPT_FILENAME"]));
			break;
			case "REQUEST_URI":
				// Typical application of REQUEST_URI is return urls, forms submitting to itselt etc. Eg:	returnUrl=".rawurlencode(t3lib_div::getIndpEnv("REQUEST_URI"))
				if (!$HTTP_SERVER_VARS["REQUEST_URI"])	{	// This is for ISS/CGI which does not have the REQUEST_URI available.
					return "/".ereg_replace("^/","",t3lib_div::getIndpEnv("SCRIPT_NAME")).
						($HTTP_SERVER_VARS["QUERY_STRING"]?"?".$HTTP_SERVER_VARS["QUERY_STRING"]:"");
				} else return $HTTP_SERVER_VARS["REQUEST_URI"];
			break;
			case "PATH_INFO":
					// $HTTP_SERVER_VARS["PATH_INFO"]!=$HTTP_SERVER_VARS["SCRIPT_NAME"] is necessary because some servers (Windows/CGI) are seen to set PATH_INFO equal to script_name
					// Further, there must be at least one "/" in the path - else the PATH_INFO value does not make sense.
					// IF "PATH_INFO" never works for our purpose in Typo3 with CGI-servers, then 'php_sapi_name()=="cgi"' might be a better check. Right now strcmp($HTTP_SERVER_VARS["PATH_INFO"],t3lib_div::getIndpEnv("SCRIPT_NAME")) will always return false for CGI-versions, but that is only as long as SCRIPT_NAME is set equal to PATH_INFO because of php_sapi_name()=="cgi" (see above)
//				if (strcmp($HTTP_SERVER_VARS["PATH_INFO"],t3lib_div::getIndpEnv("SCRIPT_NAME")) && count(explode("/",$HTTP_SERVER_VARS["PATH_INFO"]))>1)	{
				if (php_sapi_name()!="cgi")	{
					return $HTTP_SERVER_VARS["PATH_INFO"];
				} else return "";
			break;
				// These are let through without modification
			case "REMOTE_ADDR":
			case "REMOTE_HOST":
			case "HTTP_REFERER":
			case "HTTP_HOST":
			case "HTTP_USER_AGENT":
			case "HTTP_ACCEPT_LANGUAGE":
			case "QUERY_STRING":
				return $HTTP_SERVER_VARS[$getEnvName];
			break;
			case "TYPO3_DOCUMENT_ROOT":
				// Some CGI-versions (LA13CGI) and mod-rewrite rules on MODULE versions will deliver a "wrong" DOCUMENT_ROOT (according to our description). Further various aliases/mod_rewrite rules can disturb this as well.
				// Therefore the DOCUMENT_ROOT is now always calculated as the SCRIPT_FILENAME minus the end part shared with SCRIPT_NAME.
				$SFN = t3lib_div::getIndpEnv("SCRIPT_FILENAME");
				$SN_A = explode("/",strrev(t3lib_div::getIndpEnv("SCRIPT_NAME")));
				$SFN_A = explode("/",strrev($SFN));
				$acc=array();
				while(list($kk,$vv)=each($SN_A))	{
					if (!strcmp($SFN_A[$kk],$vv))	{
						$acc[]=$vv;
					} else break;
				}
				$commonEnd=strrev(implode("/",$acc));
				if (strcmp($commonEnd,""))		$DR = substr($SFN,0,-(strlen($commonEnd)+1));
				return $DR;
			break;
			case "TYPO3_HOST_ONLY":
				$p=explode(":",$HTTP_SERVER_VARS["HTTP_HOST"]);
				return $p[0];
			break;
			case "TYPO3_PORT":
				$p=explode(":",$HTTP_SERVER_VARS["HTTP_HOST"]);
				return $p[1];
			break;
			case "TYPO3_REQUEST_HOST":
				return "http".($HTTP_SERVER_VARS["SSL_SESSION_ID"]?"s":"")."://".	// I hope this: ($HTTP_SERVER_VARS["SSL_SESSION_ID"]?"s":"") - is sufficient to detect https...
					$HTTP_SERVER_VARS["HTTP_HOST"];
			break;
			case "TYPO3_REQUEST_URL":
				return t3lib_div::getIndpEnv("TYPO3_REQUEST_HOST").t3lib_div::getIndpEnv("REQUEST_URI");
			break;
			case "TYPO3_REQUEST_SCRIPT":
				return t3lib_div::getIndpEnv("TYPO3_REQUEST_HOST").t3lib_div::getIndpEnv("SCRIPT_NAME");
			break;
			case "TYPO3_REQUEST_DIR":
				return t3lib_div::getIndpEnv("TYPO3_REQUEST_HOST").t3lib_div::dirname(t3lib_div::getIndpEnv("SCRIPT_NAME"))."/";
			break;
			case "TYPO3_SITE_URL":
				if (defined("PATH_thisScript") && defined("PATH_site"))	{
					$lPath = substr(dirname(PATH_thisScript),strlen(PATH_site))."/";
					$url = t3lib_div::getIndpEnv("TYPO3_REQUEST_DIR");
					$siteUrl = substr($url,0,-strlen($lPath));
					if (substr($siteUrl,-1)!="/")	$siteUrl.="/";
					return $siteUrl;
				} else return "";

			break;
			case "_ARRAY":
				$out=array();
					// Here, list ALL possible keys to this function for debug display.
				$envTestVars = t3lib_div::trimExplode(",","
					HTTP_HOST,
					TYPO3_HOST_ONLY,
					TYPO3_PORT,
					PATH_INFO,
					QUERY_STRING,
					REQUEST_URI,
					HTTP_REFERER,
					TYPO3_REQUEST_HOST,
					TYPO3_REQUEST_URL,
					TYPO3_REQUEST_SCRIPT,
					TYPO3_REQUEST_DIR,
					TYPO3_SITE_URL,
					SCRIPT_NAME,
					TYPO3_DOCUMENT_ROOT,
					SCRIPT_FILENAME,
					REMOTE_ADDR,
					REMOTE_HOST,
					HTTP_USER_AGENT,
					HTTP_ACCEPT_LANGUAGE",1);
				reset($envTestVars);
				while(list(,$v)=each($envTestVars))	{
					$out[$v]=t3lib_div::getIndpEnv($v);
				}
				reset($out);
				return $out;
			break;
		}
	}

	
	


















	
	
	## *****************************
	## TYPO3 SPECIFIC FUNCTIONS	
	## *****************************
	
	/**
	 * Returns the absolute filename of $filename.
	 * If $onlyRelative is set (which it is by default), then only return values relative to the current PATH_site is accepted.
	 * Decodes the prefix EXT: for Typo3 Extensions.
	 */
	function getFileAbsFileName($filename,$onlyRelative=1)	{
		if (!strcmp($filename,""))		return "";

		if (substr($filename,0,4)=="EXT:")	{	// extension
			list($extKey,$local) = explode("/",substr($filename,4),2);
			$filename="";
			if (strcmp($extKey,"") && t3lib_extMgm::isLoaded($extKey) && strcmp($local,""))	{
				$filename = t3lib_extMgm::extPath($extKey).$local;
			}
		} elseif (!t3lib_div::isAbsPath($filename))	{	// relative. Prepended with PATH_site
			$filename=PATH_site.$filename;
		} elseif ($onlyRelative && !t3lib_div::isFirstPartOfStr($filename,PATH_site)) {	// absolute, but set to blank if not allowed
			$filename="";
		}
		if (strcmp($filename,"") && t3lib_div::validPathStr($filename))	{	// checks backpath.
			return $filename;
		}
	}
	
	/**
	 * Returns true if no "//", ".." or "\" is in the $theFile
	 * This should make sure that the path is not pointing "backwards" and further doesn't contain double/back slashes.
	 * So it's compatible with  the UNIX style path strings valid for TYPO3 internally.
	 */
	function validPathStr($theFile)	{
		if (!strstr($theFile,"//") && !strstr($theFile,"..") && !strstr($theFile,"\\"))	return true;
	}

	/**
	 * Checks if the $path is absolute or relative (detecting either "/" or "x:/" as first part of string) and returns true if so.
	 */
	function isAbsPath($path)	{
		return TYPO3_OS=="WIN" ? substr($path,1,2)==":/" :  substr($path,0,1)=="/";
	}

	/**
	 * Returns true if the path is absolute, without backpath ".." and within the PATH_site OR within the lockRootPath
	 */
	function isAllowedAbsPath($path)	{
		if (t3lib_div::isAbsPath($path) &&
			t3lib_div::validPathStr($path) &&
				(	t3lib_div::isFirstPartOfStr($path,PATH_site) 
					|| 
					($GLOBALS["TYPO3_CONF_VARS"]["BE"]["lockRootPath"] && t3lib_div::isFirstPartOfStr($path,$GLOBALS["TYPO3_CONF_VARS"]["BE"]["lockRootPath"]))
				)
			)	return true;
	}
	
	/**
	 * Verifies the input filename againts the "fileDenyPattern". Returns true if OK.
	 */
	function verifyFilenameAgainstDenyPattern($filename)	{
		if (strcmp($filename,"") && strcmp($GLOBALS["TYPO3_CONF_VARS"]["BE"]["fileDenyPattern"],""))	{
			$result = eregi($GLOBALS["TYPO3_CONF_VARS"]["BE"]["fileDenyPattern"],$filename);
			if ($result)	return false;	// so if a matching filename is found, return false;
		}
		return true;
	}
	
	/**
	 * standard authentication code - can't remember what it's used for.
	 */
	function stdAuthCode($uid_or_record,$fields="")	{
		if (is_array($uid_or_record))	{	
			$recCopy_temp=array();
			if ($fields)	{
				$fieldArr = t3lib_div::trimExplode(",",$fields,1);
				reset($fieldArr);
				while(list($k,$v)=each($fieldArr))	{
					$recCopy_temp[$k]=$recCopy[$v];
				}
			} else {
				$recCopy_temp=$recCopy;
			}
			$preKey = implode("|",$recCopy_temp);
		} else {
			$preKey = $uid_or_record;
		}

		$authCode = $preKey."||".$GLOBALS["TYPO3_CONF_VARS"]["SYS"]["encryptionKey"];
		$authCode = substr(md5($authCode),0,8);
		return $authCode;
	}	

	/**
	 * Loads the $TCA (Table Configuration Array) for the $table 
	 *
	 * Requirements:
	 * 1) must be configured table (the ctrl-section configured), 
	 * 2) columns must not be an array (which it is always if whole table loaded), and
	 * 3) there is a value for dynamicConfigFile (filename in typo3conf)
	 */
	function loadTCA($table)	{
		global $TCA,$LANG_GENERAL_LABELS;
		if (isset($TCA[$table]) && !is_array($TCA[$table]["columns"]) && $TCA[$table]["ctrl"]["dynamicConfigFile"])	{
			if (!strcmp(substr($TCA[$table]["ctrl"]["dynamicConfigFile"],0,6),"T3LIB:"))	{
				include(PATH_t3lib."stddb/".substr($TCA[$table]["ctrl"]["dynamicConfigFile"],6));
			} elseif (t3lib_div::isAbsPath($TCA[$table]["ctrl"]["dynamicConfigFile"]) && @is_file($TCA[$table]["ctrl"]["dynamicConfigFile"]))	{	// Absolute path...
				include($TCA[$table]["ctrl"]["dynamicConfigFile"]);
			} else include(PATH_typo3conf.$TCA[$table]["ctrl"]["dynamicConfigFile"]);
		}
	}

	/**
	 * Calls a userdefined function/method in class
	 */
	function callUserFunction($funcName,&$params,&$ref,$checkPrefix="user_",$silent=0)	{
		if ($checkPrefix && 
			!t3lib_div::isFirstPartOfStr(trim($funcName),$checkPrefix) &&
			!t3lib_div::isFirstPartOfStr(trim($funcName),"tx_")
			)	{
			if (!$silent)	debug("Function '".$funcName."' was not prepended with '".$checkPrefix."'",1);
			return $content;
		}
		$parts = explode("->",$funcName);
		if (count($parts)==2)	{	// Class
			if (class_exists($parts[0]))	{
				$classObj = new $parts[0];
				if (method_exists($classObj, $parts[1]))	{
				 	$content = call_user_method($parts[1], $classObj, $params, $ref);
				} else {
					if (!$silent)	debug("<strong>ERROR:</strong> No method name '".$parts[1]."' in class ".$parts[0],1);
				}
			} else {
				if (!$silent)	debug("<strong>ERROR:</strong> No class named: ".$parts[0],1);
			}
		} else {	// Function
			if (function_exists($funcName))	{
			 	$content = call_user_func($funcName, $params, $ref);
			} else {
				if (!$silent)	debug("<strong>ERROR:</strong> No function named: ".$funcName,1);
			}
		}
		return $content;
	}

	/**
	 * Make instance of class
	 * Takes the class-extensions API of Typo3 into account
	 * Please USE THIS instead of the PHP "new" keyword. Eg. "$obj = new myclass;" should be "$obj = t3lib_div::makeInstance("myclass")" instead!
	 */
	function makeInstance($className)	{
		$className = class_exists("ux_".$className)?"ux_".$className:$className;
		return new $className;
	}

	/**
	 * Return classname for new instance
	 * Takes the class-extensions API of Typo3 into account
	 */
	function makeInstanceClassName($className)	{
		return class_exists("ux_".$className)?"ux_".$className:$className;
	}

	/**
	 * Simple substitute for the PHP function mail().
	 * The fifth parameter, $enc, will allow you to specify "base64" encryption for the output (set $enc=base64)
	 * Further the output has the charset set to ISO-8859-1 by default.
	 */
	function plainMailEncoded($email,$subject,$message,$headers="",$enc="",$charset="ISO-8859-1",$dontEncodeSubject=0)	{
		switch((string)$enc)	{
			case "base64":
				$headers=trim($headers).chr(10).
				'Mime-Version: 1.0
				Content-Type: text/plain; charset="'.$charset.'"
				Content-Transfer-Encoding: base64';
				
				$message=trim(chunk_split(base64_encode($message.chr(10)))).chr(10);	// Adding chr(10) because I think MS outlook 2002 wants it... may be removed later again.

				if (!$dontEncodeSubject)	$subject="=?".$charset."?B?".base64_encode($subject)."?=";
			break;
			case "quoted-printable":
				$headers=trim($headers).chr(10).
				'Mime-Version: 1.0
				Content-Type: text/plain; charset="'.$charset.'"
				Content-Transfer-Encoding: quoted-printable';
				
				$message=t3lib_div::quoted_printable($message);

				if (!$dontEncodeSubject)	$subject="=?".$charset."?Q?".trim(t3lib_div::quoted_printable(ereg_replace("[[:space:]]","_",$subject),1000))."?=";
			break;
			case "8bit":
				$headers=trim($headers).chr(10).
				'Mime-Version: 1.0
				Content-Type: text/plain; charset="'.$charset.'"
				Content-Transfer-Encoding: 8bit';
			break;
		}
		$headers=trim(implode(chr(10),t3lib_div::trimExplode(chr(10),$headers,1)));	// make sure no empty lines are there.
#debug(array($email,$subject,$message,$headers));

		mail($email,$subject,$message,$headers);
	}

	/**
	 * Implementation of quoted-printable encode. 
	 * This functions is buggy. It seems that in the part where the lines are breaked every 76th character, that it fails if the break happens right in a quoted_printable encode character!
	 * (Originally taken from class.t3lib_htmlmail.php - which may be updated if this function should ever be improved!
	 * See RFC 1521, section 5.1 Quoted-Printable Content-Transfer-Encoding
	 */
	function quoted_printable($string,$maxlen=76)	{
		$newString = "";
		$theLines = explode(chr(10),$string);	// Break lines. Doesn't work with mac eol's which seems to be 13. But 13-10 or 10 will work
		while (list(,$val)=each($theLines))	{
			$val = ereg_replace(chr(13)."$","",$val);		// removes possible character 13 at the end of line
			
			$newVal = "";
			$theValLen = strlen($val);
			$len = 0;
			for ($index=0;$index<$theValLen;$index++)	{
				$char = substr($val,$index,1);
				$ordVal =Ord($char);
				if ($len>($maxlen-4) || ($len>(($maxlen-10)-4)&&$ordVal==32))	{
					$len=0;
					$newVal.="=".chr(13).chr(10);
				}
				if (($ordVal>=33 && $ordVal<=60) || ($ordVal>=62 && $ordVal<=126) || $ordVal==9 || $ordVal==32)	{
					$newVal.=$char;
					$len++;
				} else {
					$newVal.=sprintf("=%02X",$ordVal);
					$len+=3;
				}
			}
			$newVal = ereg_replace(chr(32)."$","=20",$newVal);		// replaces a possible SPACE-character at the end of a line
			$newVal = ereg_replace(chr(9)."$","=09",$newVal);		// replaces a possible TAB-character at the end of a line
			$newString.=$newVal.chr(13).chr(10);
		}
		return $newString;
	}

	/**
	 * Takes a clear-text message body for a plain text email, finds all "http://" links and if they are longer than 76 chars they are converted to a shorter URL with a hash parameter. The real parameter is stored in the database and the hash-parameter/URL will be redirected to the real parameter when the link is clicked.
	 * This function is about preserving long links in messages.
	 */
	function substUrlsInPlainText($message,$urlmode="76",$index_script_url="")	{
			// Substitute URLs with shorter links:
		$urlSplit=explode("http://",$message);
		reset($urlSplit);
		while(list($c,$v)=each($urlSplit))	{
			if ($c)	{
				$newParts = split("[[:space:]]|\)|\(",$v,2);
				$newURL="http://".$newParts[0];
#				if (!strcmp($urlSplit[$c-1],"") || ereg("[[:space:]]$",$urlSplit[$c-1]))	{
					switch((string)$urlmode)	{
						case "all":
							$newURL=t3lib_div::makeRedirectUrl($newURL,0,$index_script_url);
						break;
						case "76":
							$newURL=t3lib_div::makeRedirectUrl($newURL,76,$index_script_url);
						break;
					}
#				}
				$urlSplit[$c]=$newURL.substr($v,strlen($newParts[0]));
			}
		}
#debug($urlSplit);
		$message=implode("",$urlSplit);
		return $message;
	}

	/**
	 * Subfunction for substUrlsInPlainText() above.
	 */
	function makeRedirectUrl($inUrl,$l=0,$index_script_url="")	{
		if (strlen($inUrl)>$l)	{
			$md5=substr(md5($inUrl),0,20);
			$query="SELECT md5hash FROM cache_md5params WHERE md5hash='".$md5."'";
			$res=mysql(TYPO3_db,$query);
			if (!mysql_num_rows($res))	{
				$query="INSERT INTO cache_md5params (md5hash,tstamp,type,params) VALUES ('".$md5."',".time().",2,'".addslashes($inUrl)."')";
				$res=mysql(TYPO3_db,$query);
			}
			$inUrl=($index_script_url ? $index_script_url : t3lib_div::getIndpEnv("TYPO3_REQUEST_DIR")).
				"?RDCT=".$md5;
		}
		return $inUrl;
	}

	/**
	 * Function to compensate for FreeType2 96 dpi
	 */
	function freetypeDpiComp($font_size)	{
		$dpi = intval($GLOBALS["TYPO3_CONF_VARS"]["GFX"]["TTFdpi"]);
		if ($dpi!=72)	$font_size = $font_size/$dpi*72;
		return $font_size;
	}
}


?>