<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 1999-2003 Kasper Skrhj (kasper@typo3.com)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Extension Management functions
 *
 * @author	Kasper Skrhj <kasper@typo3.com>
 */

class t3lib_extMgm {
	
	function isLocalconfWritable()	{
		return is_writeable(PATH_typo3conf) && is_writeable(PATH_typo3conf."localconf.php");
	}
	function cannotCacheFilesWritable($cacheFilePrefix)	{
		$error=array();
		if (!@is_writeable(PATH_typo3conf))	{
			$error[]=PATH_typo3conf;
		}
		if (@is_file(PATH_typo3conf.$cacheFilePrefix."_ext_localconf.php") && 
			!@is_writeable(PATH_typo3conf.$cacheFilePrefix."_ext_localconf.php"))	{
				$error[]=PATH_typo3conf.$cacheFilePrefix."_ext_localconf.php";
		}
		if (@is_file(PATH_typo3conf.$cacheFilePrefix."_ext_tables.php") && 
			!@is_writeable(PATH_typo3conf.$cacheFilePrefix."_ext_tables.php"))	{
				$error[]=PATH_typo3conf.$cacheFilePrefix."_ext_tables.php";
		}
		return implode(", ",$error);
	}
	function currentCacheFiles()	{
		global $TYPO3_LOADED_EXT;
		
		if ($TYPO3_LOADED_EXT["_CACHEFILE"])	{
			if (t3lib_extMgm::isCacheFilesAvailable($TYPO3_LOADED_EXT["_CACHEFILE"]))	{
				return array(
					PATH_typo3conf.$TYPO3_LOADED_EXT["_CACHEFILE"]."_ext_localconf.php",
					PATH_typo3conf.$TYPO3_LOADED_EXT["_CACHEFILE"]."_ext_tables.php"
				);
			}
		}
	}
	
	function typo3_loadExtensions()	{
		global $TYPO3_CONF_VARS;

		$rawExtList = $TYPO3_CONF_VARS["EXT"]["requiredExt"].",".$TYPO3_CONF_VARS["EXT"]["extList"];

		$extensions = array();
		if ($rawExtList)	{
			$cacheFilePrefix = "temp_CACHED";

			if (intval($TYPO3_CONF_VARS["EXT"]["extCache"])==1)	$cacheFilePrefix.= "_ps".substr(t3lib_div::shortMD5(PATH_site."|".$GLOBALS["TYPO_VERSION"]),0,4);
			if (intval($TYPO3_CONF_VARS["EXT"]["extCache"])==2)	$cacheFilePrefix.= "_".t3lib_div::shortMD5($rawExtList);

				// If cache files available, set cache file prefix and return:
			if ($TYPO3_CONF_VARS["EXT"]["extCache"] && t3lib_extMgm::isCacheFilesAvailable($cacheFilePrefix))	{
					// Return cache file prefix:
				$extensions["_CACHEFILE"] = $cacheFilePrefix;
			} else {
				// ... but if not, configure...
				$temp_extensions = array_unique(t3lib_div::trimExplode(",",$rawExtList,1));
				while(list(,$temp_extKey)=each($temp_extensions))	{
					if (@is_dir(PATH_site."typo3conf/ext/".$temp_extKey))	{
						$extensions[$temp_extKey]=array("type"=>"L","siteRelPath"=>"typo3conf/ext/".$temp_extKey."/","typo3RelPath"=>"../typo3conf/ext/".$temp_extKey."/");
					} elseif (@is_dir(PATH_site.TYPO3_mainDir."ext/".$temp_extKey))	{
						$extensions[$temp_extKey]=array("type"=>"G","siteRelPath"=>TYPO3_mainDir."ext/".$temp_extKey."/","typo3RelPath"=>"ext/".$temp_extKey."/");
					} elseif (@is_dir(PATH_site.TYPO3_mainDir."sysext/".$temp_extKey))	{
						$extensions[$temp_extKey]=array("type"=>"S","siteRelPath"=>TYPO3_mainDir."sysext/".$temp_extKey."/","typo3RelPath"=>"sysext/".$temp_extKey."/");
					}
					
					$files = t3lib_div::trimExplode(",","
						ext_localconf.php,
						ext_tables.php,
						ext_tables.sql,
						ext_tables_static+adt.sql,
						ext_typoscript_constants.txt,
						ext_typoscript_editorcfg.txt,
						ext_typoscript_setup.txt
					",1);
					reset($files);
					while(list(,$fName)=each($files))	{
						$temp_filename = PATH_site.$extensions[$temp_extKey]["siteRelPath"].trim($fName);
						if (is_array($extensions[$temp_extKey]) && @is_file($temp_filename))	{
							$extensions[$temp_extKey][$fName]=$temp_filename;
						}
					}
				}
				unset($extensions["_CACHEFILE"]);
				
				
				// write cache?
				if ($TYPO3_CONF_VARS["EXT"]["extCache"])	{
					$wrError = t3lib_extMgm::cannotCacheFilesWritable($cacheFilePrefix);
					if ($wrError)	{
//debug("Cannot write cache files: ".$wrError.". Disabling the cache...");
						$TYPO3_CONF_VARS["EXT"]["extCache"]=0;
					} else {
						// Write cache files:
						$extensions = t3lib_extMgm::writeCacheFiles($extensions,$cacheFilePrefix);
					}
				}
			}
		}

#debug($extensions);
		return $extensions;
	}
	function _makeIncludeHeader($key,$file)	{
		return '<?php
###########################
## EXTENSION: '.$key.'
## FILE:      '.$file.'
###########################

$_EXTKEY = "'.$key.'";
$_EXTCONF = $TYPO3_CONF_VARS["EXT"]["extConf"][$_EXTKEY];

?>';
	}
	function isCacheFilesAvailable($cacheFilePrefix)	{
		return 
			@is_file(PATH_typo3conf.$cacheFilePrefix."_ext_localconf.php") && 
			@is_file(PATH_typo3conf.$cacheFilePrefix."_ext_tables.php");
	}
	function writeCacheFiles($extensions,$cacheFilePrefix)	{
			// Making cache files:
		$extensions["_CACHEFILE"] = $cacheFilePrefix;
		$cFiles=array();
		$cFiles["ext_localconf"].='<?php

$TYPO3_LOADED_EXT = unserialize(stripslashes("'.addslashes(serialize($extensions)).'"));

?>';

		reset($extensions);
		while(list($key,$conf)=each($extensions))	{
			if (is_array($conf))	{
				if ($conf["ext_localconf.php"])	{
					$cFiles["ext_localconf"].=t3lib_extMgm::_makeIncludeHeader($key,$conf["ext_localconf.php"]);
					$cFiles["ext_localconf"].=trim(t3lib_div::getUrl($conf["ext_localconf.php"]));
				}
				if ($conf["ext_tables.php"])	{
					$cFiles["ext_tables"].=t3lib_extMgm::_makeIncludeHeader($key,$conf["ext_tables.php"]);
					$cFiles["ext_tables"].=trim(t3lib_div::getUrl($conf["ext_tables.php"]));
				}
			}
		}

		t3lib_div::writeFile(PATH_typo3conf.$cacheFilePrefix."_ext_localconf.php",$cFiles["ext_localconf"]);
		t3lib_div::writeFile(PATH_typo3conf.$cacheFilePrefix."_ext_tables.php",$cFiles["ext_tables"]);

		$extensions=array();
		$extensions["_CACHEFILE"] = $cacheFilePrefix;

		return $extensions;
	}
	function allowTableOnStandardPages($table)	{
		global $PAGES_TYPES;
		
		$PAGES_TYPES["default"]["allowedTables"].=",".$table;
	}
	function insertModuleFunction($modname,$className,$classPath,$title,$MM_key="function")	{
		global $TBE_MODULES_EXT;
		$TBE_MODULES_EXT[$modname]["MOD_MENU"][$MM_key][$className]=array(
			"name" => $className,
			"path" => $classPath,
			"title" => $title,
		);
	}
	function addModule($main,$sub="",$position="",$path="")	{
		global $TBE_MODULES;
		
		if (isset($TBE_MODULES[$main]) && $sub)	{
			list($place,$modRef)=t3lib_div::trimExplode(":",$position,1);
			$mods = t3lib_div::trimExplode(",",$TBE_MODULES[$main],1);
			if (!in_array($sub,$mods))	{
				switch(strtolower($place))	{
					case "after":
					case "before":
						$pointer=0;
						reset($mods);
						while(list($k,$m)=each($mods))	{
							if (!strcmp($m,$modRef))	{
								$pointer=strtolower($place)=="after"?$k+1:$k;
							}
						}
						array_splice(
							$mods,	// The modules array
							$pointer,		// To insert one position from the end of the list
							0,		// Don't remove any items, just insert
							$sub	// Module to insert
						);
					break;
					default:
						if (strtolower($place)=="top")	{
							array_unshift($mods,$sub);
						} else {
							array_push($mods,$sub);
						}
					break;
				}
			}
			$TBE_MODULES[$main]=implode(",",$mods);
		} else {
			$TBE_MODULES[$main]=$sub;
		}

		if ($path)	{
			$TBE_MODULES["_PATHS"][$main.($sub?"_".$sub:"")]=$path;
		}
	}
	function addPlugin($itemArray,$type="list_type")	{
		global $TCA;
		t3lib_div::loadTCA("tt_content");
		if (is_array($TCA["tt_content"]["columns"]) && is_array($TCA["tt_content"]["columns"][$type]["config"]["items"]))	{
			reset($TCA["tt_content"]["columns"][$type]["config"]["items"]);
			while(list($k,$v)=each($TCA["tt_content"]["columns"][$type]["config"]["items"]))	{
				if (!strcmp($v[1],$itemArray[1]))	{
					$TCA["tt_content"]["columns"][$type]["config"]["items"][$k]=$itemArray;
					return;
				}
			}
			$TCA["tt_content"]["columns"][$type]["config"]["items"][]=$itemArray;
		}
	}
	/**
	 * Call this method to add an entry in the static template list found in sys_templates
	 * $extKey is of course the extension key
	 * $path is the path where the template files (fixed names) constants.txt, setup.txt and editorcfg.txt is found (relative to extPath, eg. "static/")
	 * $title is the title in the selector box.
	 */
	function addStaticFile($extKey,$path,$title)	{
		global $TCA;
		t3lib_div::loadTCA("sys_template");
		if ($extKey && $path && is_array($TCA["sys_template"]["columns"]))	{
			$value = str_replace(",","","EXT:".$extKey."/".$path);
			$itemArray=array(trim($title." (".$extKey.")"),$value);
			$TCA["sys_template"]["columns"]["include_static_file"]["config"]["items"][]=$itemArray;
		}
	}
	function addToInsertRecords($table)	{
		global $TCA;
		t3lib_div::loadTCA("tt_content");
		if (is_array($TCA["tt_content"]["columns"]) && isset($TCA["tt_content"]["columns"]["records"]["config"]["allowed"]))	{
			$TCA["tt_content"]["columns"]["records"]["config"]["allowed"].=",".$table;
		}
	}
	function addPageTSConfig($content)	{
		global $TYPO3_CONF_VARS;
		$TYPO3_CONF_VARS["BE"]["defaultPageTSconfig"].="\n[GLOBAL]\n".$content;
	}
	function addUserTSConfig($content)	{
		global $TYPO3_CONF_VARS;
		$TYPO3_CONF_VARS["BE"]["defaultUserTSconfig"].="\n[GLOBAL]\n".$content;
	}
	function addTypoScriptSetup($content)	{
		global $TYPO3_CONF_VARS;
		$TYPO3_CONF_VARS["FE"]["defaultTypoScript_setup"].="\n[GLOBAL]\n".$content;
	}
	function addTypoScriptConstants($content)	{
		global $TYPO3_CONF_VARS;
		$TYPO3_CONF_VARS["FE"]["defaultTypoScript_constants"].="\n[GLOBAL]\n".$content;
	}
	function getCN($key)	{
		return substr($key,0,5)=="user_" ? 'user_'.str_replace("_","",substr($key,5)) : 'tx_'.str_replace("_","",$key);
	}
	function addPItoST43($key,$classFile="",$prefix="",$type="list_type",$cached=0)	{
		global $TYPO3_LOADED_EXT;
		$classFile = $classFile ? $classFile : 'pi/class.tx_'.str_replace("_","",$key).$prefix.'.php';
		$cN = t3lib_extMgm::getCN($key);
		
			// General plugin:
		if ($cached)	{
			$pluginContent = trim('
includeLibs.'.$cN.$prefix.' = '.$TYPO3_LOADED_EXT[$key]["siteRelPath"].$classFile.'
plugin.'.$cN.$prefix.' = USER
plugin.'.$cN.$prefix.' { 
  userFunc = '.$cN.$prefix.'->main
}');	
		} else {
			$pluginContent = trim('
plugin.'.$cN.$prefix.' = USER_INT
plugin.'.$cN.$prefix.' { 
  includeLibs = '.$TYPO3_LOADED_EXT[$key]["siteRelPath"].$classFile.'
  userFunc = '.$cN.$prefix.'->main
}');	
		}
		t3lib_extMgm::addTypoScript($key,"setup",'
# Setting '.$key.' plugin TypoScript
'.$pluginContent);

			// After ST43:
		switch($type)	{
			case "list_type":
				$addLine = 'tt_content.list.20.'.$key.$prefix.' = < plugin.'.$cN.$prefix;
			break;
			case "menu_type":
				$addLine = 'tt_content.menu.20.'.$key.$prefix.' = < plugin.'.$cN.$prefix;
			break;
			case "splash_layout":
				$addLine = 'tt_content.splash.'.$key.$prefix.' = < plugin.'.$cN.$prefix;
			break;
			case "CType":
				$addLine = trim('
tt_content.'.$key.$prefix.' = COA
tt_content.'.$key.$prefix.' { 
	10 = < lib.stdheader
	20 = < plugin.'.$cN.$prefix.'
}
				');
			break;
			case "header_layout":
				$addLine = 'lib.stdheader.10.'.$key.$prefix.' = < plugin.'.$cN.$prefix;
			break;
			case "includeLib":
				$addLine = 'page.1000 = < plugin.'.$cN.$prefix;
			break;
			default:
				$addLine = '';
			break;
		}
		if ($addLine)	{
			t3lib_extMgm::addTypoScript($key,"setup",'
# Setting '.$key.' plugin TypoScript
'.$addLine.'
',43);
		}
	}
/*	function addSingleView($key,$classFile,$prefix,$table,$cached=0)	{
		global $TYPO3_LOADED_EXT;
		$classFile = $classFile ? $classFile : 'pi/class.tx_'.str_replace("_","",$key).$prefix.'.php';
		$cN = t3lib_extMgm::getCN($key);
		
			// General plugin:
		if ($cached)	{
			$pluginContent = trim('
includeLibs.'.$cN.$prefix.' = '.$TYPO3_LOADED_EXT[$key]["siteRelPath"].$classFile.'
'.$table.' = USER
'.$table.'.userFunc = '.$cN.$prefix.'->singleView
');	
		} else {
			$pluginContent = trim('
'.$table.' = USER_INT
'.$table.' { 
  includeLibs = '.$TYPO3_LOADED_EXT[$key]["siteRelPath"].$classFile.'
  userFunc = '.$cN.$prefix.'->singleView
}');	
		}
		t3lib_extMgm::addTypoScript($key,"setup",'
# Setting rendering for records from "'.$table.'"
'.$pluginContent);
	}*/
	function addPluginTS()	{
	}
	function addTypoScript($key,$type,$content,$afterStaticUid=0)	{
		global $TYPO3_CONF_VARS;
		
		if ($type=="setup" || $type=="editorcfg" || $type=="constants")		{
			$content = "

[GLOBAL]
#############################################
## TypoScript added by extension '".$key."'
#############################################

".$content;
			if ($afterStaticUid)	{
				$TYPO3_CONF_VARS["FE"]["defaultTypoScript_".$type."."][$afterStaticUid].=$content;
				if ($afterStaticUid==43)	{	// If "content (default)" is targeted, also add to other "content rendering templates", eg. css_styled_content
					$TYPO3_CONF_VARS["FE"]["defaultTypoScript_".$type."."]["cssstyledcontent/static/"].=$content;
				}
			} else {
				$TYPO3_CONF_VARS["FE"]["defaultTypoScript_".$type].=$content;
			}
		}
	}
	function addTCAcolumns($table,$columnArray,$addTofeInterface=0)	{
		global $TCA;
		t3lib_div::loadTCA($table);	
		if (is_array($columnArray) && is_array($TCA[$table]) && is_array($TCA[$table]["columns"]))	{
			$TCA[$table]["columns"] = array_merge($TCA[$table]["columns"],$columnArray);	// Candidate for t3lib_div::array_merge() if integer-keys will some day make trouble...
			if ($addTofeInterface)	$TCA[$table]["feInterface"]["fe_admin_fieldList"].=",".implode(",",array_keys($columnArray));
		}
	}
	function addToAllTCAtypes($table,$str)	{
		global $TCA;
		t3lib_div::loadTCA($table);	
		if (trim($str) && is_array($TCA[$table]) && is_array($TCA[$table]["types"]))	{
			reset($TCA[$table]["types"]);
			while(list($k)=each($TCA[$table]["types"]))	{
				$TCA[$table]["types"][$k]["showitem"].=", ".trim($str);
			}
		}
	}
	function extPath($key,$script="")	{
		global $TYPO3_LOADED_EXT;
		if (!isset($TYPO3_LOADED_EXT[$key]))	die("Typo3 Fatal Error: Extension key '".$key."' was NOT loaded!");
		return PATH_site.$TYPO3_LOADED_EXT[$key]["siteRelPath"].$script;
	}
	function extRelPath($key)	{
		global $TYPO3_LOADED_EXT;
		if (!isset($TYPO3_LOADED_EXT[$key]))	die("Typo3 Fatal Error: Extension key '".$key."' was NOT loaded!");
		return $TYPO3_LOADED_EXT[$key]["typo3RelPath"];
	}
	function siteRelPath($key)	{
		return substr(t3lib_extMgm::extPath($key),strlen(PATH_site));
	}
	function isLoaded($key,$exitOnError=0)	{
		global $TYPO3_LOADED_EXT;
		if ($exitOnError && !isset($TYPO3_LOADED_EXT[$key]))	die("Fatal Error: Extension '".$key."' was not loaded.");
		return isset($TYPO3_LOADED_EXT[$key]);
	}
}


?>